@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,32 +9,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "newdialog.h"
 #include "ui_newdialog.h"
 
-#include <coreplugin/coreconstants.h>
-#include <coreplugin/documentmanager.h>
+#include <coreplugin/icontext.h>
 #include <coreplugin/icore.h>
+#include <utils/algorithm.h>
 #include <utils/qtcassert.h>
+#include <utils/utilsicons.h>
 
 #include <QAbstractProxyModel>
 #include <QDebug>
@@ -45,13 +41,15 @@
 #include <QPushButton>
 #include <QSortFilterProxyModel>
 #include <QStandardItem>
+#include <QTimer>
 
 Q_DECLARE_METATYPE(Core::IWizardFactory*)
 
-
 namespace {
 
 const int ICON_SIZE = 22;
+const char LAST_CATEGORY_KEY[] = "Core/NewDialog/LastCategory";
+const char LAST_PLATFORM_KEY[] = "Core/NewDialog/LastPlatform";
 
 class WizardFactoryContainer
 {
@@ -75,7 +73,7 @@ class PlatformFilterProxyModel : public QSortFilterProxyModel
 public:
     PlatformFilterProxyModel(QObject *parent = 0): QSortFilterProxyModel(parent) {}
 
-    void setPlatform(const QString& platform)
+    void setPlatform(Core::Id platform)
     {
         m_platform = platform;
         invalidateFilter();
@@ -89,12 +87,12 @@ public:
         QModelIndex sourceIndex = sourceModel()->index(sourceRow, 0, sourceParent);
         Core::IWizardFactory *wizard = factoryOfItem(qobject_cast<QStandardItemModel*>(sourceModel())->itemFromIndex(sourceIndex));
         if (wizard)
-            return m_platform.isEmpty() || wizard->isAvailable(m_platform);
+            return wizard->isAvailable(m_platform);
 
         return true;
     }
 private:
-    QString m_platform;
+    Core::Id m_platform;
 };
 
 class TwoLevelProxyModel : public QAbstractProxyModel
@@ -188,16 +186,21 @@ Q_DECLARE_METATYPE(WizardFactoryContainer)
 using namespace Core;
 using namespace Core::Internal;
 
-QString NewDialog::m_lastCategory = QString();
+QWidget *NewDialog::m_currentDialog = nullptr;
 
 NewDialog::NewDialog(QWidget *parent) :
     QDialog(parent),
-    m_ui(new Core::Internal::Ui::NewDialog),
+    m_ui(new Ui::NewDialog),
     m_okButton(0)
 {
+    QTC_CHECK(m_currentDialog == nullptr);
+
+    m_currentDialog = this;
+
     setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
     setWindowFlags(windowFlags());
     setAttribute(Qt::WA_DeleteOnClose);
+    ICore::registerWindow(this, Context("Core.NewDialog"));
     m_ui->setupUi(this);
     QPalette p = m_ui->frame->palette();
     p.setColor(QPalette::Window, p.color(QPalette::Base));
@@ -219,22 +222,19 @@ NewDialog::NewDialog(QWidget *parent) :
     m_ui->templatesView->setModel(m_filterProxyModel);
     m_ui->templatesView->setIconSize(QSize(ICON_SIZE, ICON_SIZE));
 
-    connect(m_ui->templateCategoryView->selectionModel(),
-            SIGNAL(currentChanged(QModelIndex,QModelIndex)),
-            this, SLOT(currentCategoryChanged(QModelIndex)));
+    connect(m_ui->templateCategoryView->selectionModel(), &QItemSelectionModel::currentChanged,
+            this, &NewDialog::currentCategoryChanged);
 
-    connect(m_ui->templatesView->selectionModel(),
-            SIGNAL(currentChanged(QModelIndex,QModelIndex)),
-            this, SLOT(currentItemChanged(QModelIndex)));
+    connect(m_ui->templatesView->selectionModel(), &QItemSelectionModel::currentChanged,
+            this, &NewDialog::currentItemChanged);
 
-    connect(m_ui->templatesView,
-            SIGNAL(doubleClicked(QModelIndex)),
-            this, SLOT(okButtonClicked()));
+    connect(m_ui->templatesView, &QListView::doubleClicked, this, &NewDialog::accept);
+    connect(m_ui->buttonBox, &QDialogButtonBox::accepted, this, &NewDialog::accept);
+    connect(m_ui->buttonBox, &QDialogButtonBox::rejected, this, &NewDialog::reject);
 
-    connect(m_okButton, SIGNAL(clicked()), this, SLOT(okButtonClicked()));
-    connect(m_ui->buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
-
-    connect(m_ui->comboBox, SIGNAL(currentIndexChanged(QString)), this, SLOT(setSelectedPlatform(QString)));
+    connect(m_ui->comboBox,
+            static_cast<void (QComboBox::*)(const QString &)>(&QComboBox::currentIndexChanged),
+            this, &NewDialog::setSelectedPlatform);
 }
 
 // Sort by category. id
@@ -242,7 +242,7 @@ static bool wizardFactoryLessThan(const IWizardFactory *f1, const IWizardFactory
 {
     if (const int cc = f1->category().compare(f2->category()))
         return cc < 0;
-    return f1->id().compare(f2->id()) < 0;
+    return f1->id().toString().compare(f2->id().toString()) < 0;
 }
 
 void NewDialog::setWizardFactories(QList<IWizardFactory *> factories,
@@ -251,7 +251,7 @@ void NewDialog::setWizardFactories(QList<IWizardFactory *> factories,
 {
     m_defaultLocation = defaultLocation;
     m_extraVariables = extraVariables;
-    qStableSort(factories.begin(), factories.end(), wizardFactoryLessThan);
+    std::stable_sort(factories.begin(), factories.end(), wizardFactoryLessThan);
 
     m_model->clear();
     QStandardItem *parentItem = m_model->invisibleRootItem();
@@ -259,28 +259,26 @@ void NewDialog::setWizardFactories(QList<IWizardFactory *> factories,
     QStandardItem *projectKindItem = new QStandardItem(tr("Projects"));
     projectKindItem->setData(IWizardFactory::ProjectWizard, Qt::UserRole);
     projectKindItem->setFlags(0); // disable item to prevent focus
-    QStandardItem *filesClassesKindItem = new QStandardItem(tr("Files and Classes"));
-    filesClassesKindItem->setData(IWizardFactory::FileWizard, Qt::UserRole);
-    filesClassesKindItem->setFlags(0); // disable item to prevent focus
+    QStandardItem *filesKindItem = new QStandardItem(tr("Files and Classes"));
+    filesKindItem->setData(IWizardFactory::FileWizard, Qt::UserRole);
+    filesKindItem->setFlags(0); // disable item to prevent focus
 
     parentItem->appendRow(projectKindItem);
-    parentItem->appendRow(filesClassesKindItem);
+    parentItem->appendRow(filesKindItem);
 
     if (m_dummyIcon.isNull())
-        m_dummyIcon = QIcon(QLatin1String(Core::Constants::ICON_NEWFILE));
+        m_dummyIcon = Utils::Icons::NEWFILE.icon();
 
-    QStringList availablePlatforms = IWizardFactory::allAvailablePlatforms();
-    m_ui->comboBox->addItem(tr("All Templates"), QString());
+    QSet<Id> availablePlatforms = IWizardFactory::allAvailablePlatforms();
+    m_ui->comboBox->addItem(tr("All Templates"), Id().toSetting());
 
-    foreach (const QString &platform, availablePlatforms) {
+    foreach (Id platform, availablePlatforms) {
         const QString displayNameForPlatform = IWizardFactory::displayNameForPlatform(platform);
-        m_ui->comboBox->addItem(tr("%1 Templates").arg(displayNameForPlatform), platform);
+        m_ui->comboBox->addItem(tr("%1 Templates").arg(displayNameForPlatform), platform.toSetting());
     }
 
-    if (!availablePlatforms.isEmpty())
-        m_ui->comboBox->setCurrentIndex(1); //First Platform
-    else
-        m_ui->comboBox->setDisabled(true);
+    m_ui->comboBox->setCurrentIndex(0); // "All templates"
+    m_ui->comboBox->setEnabled(!availablePlatforms.isEmpty());
 
     foreach (IWizardFactory *factory, factories) {
         QStandardItem *kindItem;
@@ -288,10 +286,9 @@ void NewDialog::setWizardFactories(QList<IWizardFactory *> factories,
         case IWizardFactory::ProjectWizard:
             kindItem = projectKindItem;
             break;
-        case IWizardFactory::ClassWizard:
         case IWizardFactory::FileWizard:
         default:
-            kindItem = filesClassesKindItem;
+            kindItem = filesKindItem;
             break;
         }
         addItem(kindItem, factory);
@@ -304,9 +301,18 @@ void NewDialog::showDialog()
 {
     QModelIndex idx;
 
-    if (!m_lastCategory.isEmpty())
+    QString lastPlatform = ICore::settings()->value(QLatin1String(LAST_PLATFORM_KEY)).toString();
+    QString lastCategory = ICore::settings()->value(QLatin1String(LAST_CATEGORY_KEY)).toString();
+
+    if (!lastPlatform.isEmpty()) {
+        int index = m_ui->comboBox->findData(lastPlatform);
+        if (index != -1)
+            m_ui->comboBox->setCurrentIndex(index);
+    }
+
+    if (!lastCategory.isEmpty())
         foreach (QStandardItem* item, m_categoryItems) {
-            if (item->data(Qt::UserRole) == m_lastCategory)
+            if (item->data(Qt::UserRole) == lastCategory)
                 idx = m_twoLevelProxyModel->mapToSource(m_model->indexFromItem(item));
     }
     if (!idx.isValid())
@@ -327,11 +333,15 @@ void NewDialog::showDialog()
     show();
 }
 
-QString NewDialog::selectedPlatform() const
+Id NewDialog::selectedPlatform() const
 {
-    int index = m_ui->comboBox->currentIndex();
+    const int index = m_ui->comboBox->currentIndex();
+    return Id::fromSetting(m_ui->comboBox->itemData(index));
+}
 
-    return m_ui->comboBox->itemData(index).toString();
+QWidget *NewDialog::currentDialog()
+{
+    return m_currentDialog;
 }
 
 bool NewDialog::event(QEvent *event)
@@ -348,6 +358,9 @@ bool NewDialog::event(QEvent *event)
 
 NewDialog::~NewDialog()
 {
+    QTC_CHECK(m_currentDialog != nullptr);
+    m_currentDialog = nullptr;
+
     delete m_ui;
 }
 
@@ -407,8 +420,9 @@ void NewDialog::currentItemChanged(const QModelIndex &index)
     if (const IWizardFactory *wizard = factoryOfItem(cat)) {
         QString desciption = wizard->description();
         QStringList displayNamesForSupportedPlatforms;
-        foreach (const QString &platform, wizard->supportedPlatforms())
+        foreach (Id platform, wizard->supportedPlatforms())
             displayNamesForSupportedPlatforms << IWizardFactory::displayNameForPlatform(platform);
+        Utils::sort(displayNamesForSupportedPlatforms);
         if (!Qt::mightBeRichText(desciption))
             desciption.replace(QLatin1Char('\n'), QLatin1String("<br>"));
         desciption += QLatin1String("<br><br><b>");
@@ -417,7 +431,7 @@ void NewDialog::currentItemChanged(const QModelIndex &index)
         else
             desciption += tr("Supported Platforms")
                     + QLatin1String("</b>: <tt>")
-                    + displayNamesForSupportedPlatforms.join(QLatin1String(" "))
+                    + displayNamesForSupportedPlatforms.join(QLatin1Char(' '))
                     + QLatin1String("</tt>");
 
         m_ui->templateDescription->setHtml(desciption);
@@ -430,7 +444,7 @@ void NewDialog::currentItemChanged(const QModelIndex &index)
         }
 
     } else {
-        m_ui->templateDescription->setText(QString());
+        m_ui->templateDescription->clear();
     }
     updateOkButton();
 }
@@ -440,37 +454,29 @@ void NewDialog::saveState()
     QModelIndex idx = m_ui->templateCategoryView->currentIndex();
     QStandardItem *currentItem = m_model->itemFromIndex(m_twoLevelProxyModel->mapToSource(idx));
     if (currentItem)
-        m_lastCategory = currentItem->data(Qt::UserRole).toString();
+        ICore::settings()->setValue(QLatin1String(LAST_CATEGORY_KEY),
+                                    currentItem->data(Qt::UserRole));
+    ICore::settings()->setValue(QLatin1String(LAST_PLATFORM_KEY), m_ui->comboBox->currentData());
 }
 
-void NewDialog::okButtonClicked()
+static void runWizard(IWizardFactory *wizard, const QString &defaultLocation, Id platform,
+                      const QVariantMap &variables)
 {
+    QString path = wizard->runPath(defaultLocation);
+    wizard->runWizard(path, ICore::dialogParent(), platform, variables);
+}
+
+void NewDialog::accept()
+{
+    saveState();
     if (m_ui->templatesView->currentIndex().isValid()) {
-        hide();
-        saveState();
-
         IWizardFactory *wizard = currentWizardFactory();
-        QTC_ASSERT(wizard, accept(); return);
-        QString path = m_defaultLocation;
-        if (path.isEmpty()) {
-            switch (wizard->kind()) {
-            case IWizardFactory::ProjectWizard:
-                // Project wizards: Check for projects directory or
-                // use last visited directory of file dialog. Never start
-                // at current.
-                path = DocumentManager::useProjectsDirectory() ?
-                           DocumentManager::projectsDirectory() :
-                           DocumentManager::fileDialogLastVisitedDirectory();
-                break;
-            default:
-                path = DocumentManager::fileDialogInitialDirectory();
-                break;
-            }
+        if (QTC_GUARD(wizard)) {
+            QTimer::singleShot(0, std::bind(&runWizard, wizard, m_defaultLocation,
+                                            selectedPlatform(), m_extraVariables));
         }
-        wizard->runWizard(path, ICore::dialogParent(), selectedPlatform(), m_extraVariables);
-
-        close();
     }
+    QDialog::accept();
 }
 
 void NewDialog::reject()
