@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -39,7 +34,10 @@
 
 #include <aggregation/aggregate.h>
 #include <coreplugin/coreplugin.h>
+
+#include <utils/qtcassert.h>
 #include <utils/theme/theme.h>
+#include <utils/fancylineedit.h>
 
 #include <QDir>
 #include <QFrame>
@@ -58,36 +56,35 @@ using namespace Utils;
 namespace Core {
 namespace Internal {
 
-class WideEnoughLineEdit : public QLineEdit {
+class WideEnoughLineEdit : public Utils::FancyLineEdit
+{
     Q_OBJECT
+
 public:
-    WideEnoughLineEdit(QWidget *parent):QLineEdit(parent){
-        connect(this, SIGNAL(textChanged(QString)),
-                this, SLOT(updateGeometry()));
+    WideEnoughLineEdit(QWidget *parent) : Utils::FancyLineEdit(parent)
+    {
+        setFiltering(true);
+        setPlaceholderText(QString());
+        connect(this, &QLineEdit::textChanged, this, &QLineEdit::updateGeometry);
+
     }
-    ~WideEnoughLineEdit(){}
-    QSize sizeHint() const {
+
+    QSize sizeHint() const
+    {
         QSize sh = QLineEdit::minimumSizeHint();
         sh.rwidth() += qMax(25 * fontMetrics().width(QLatin1Char('x')),
                             fontMetrics().width(text()));
         return sh;
     }
-public slots:
-    void updateGeometry() { QLineEdit::updateGeometry(); }
 };
 
-} // namespace Internal
-} // namespace Core
-
-using namespace Core;
-using namespace Core::Internal;
-
 SearchResultWidget::SearchResultWidget(QWidget *parent) :
     QWidget(parent),
     m_count(0),
     m_preserveCaseSupported(true),
     m_isShowingReplaceUI(false),
-    m_searchAgainSupported(false)
+    m_searchAgainSupported(false),
+    m_replaceSupported(false)
 {
     QVBoxLayout *layout = new QVBoxLayout(this);
     layout->setMargin(0);
@@ -104,11 +101,24 @@ SearchResultWidget::SearchResultWidget(QWidget *parent) :
         topWidget->setLineWidth(1);
     }
     topWidget->setAutoFillBackground(true);
-    QHBoxLayout *topLayout = new QHBoxLayout(topWidget);
-    topLayout->setMargin(2);
+    auto topLayout = new QVBoxLayout(topWidget);
+    topLayout->setContentsMargins(2, 2, 2, 2);
+    topLayout->setSpacing(2);
     topWidget->setLayout(topLayout);
     layout->addWidget(topWidget);
 
+    auto topFindWidget = new QWidget(topWidget);
+    auto topFindLayout = new QHBoxLayout(topFindWidget);
+    topFindLayout->setMargin(0);
+    topFindWidget->setLayout(topFindLayout);
+    topLayout->addWidget(topFindWidget);
+
+    m_topReplaceWidget = new QWidget(topWidget);
+    auto topReplaceLayout = new QHBoxLayout(m_topReplaceWidget);
+    topReplaceLayout->setMargin(0);
+    m_topReplaceWidget->setLayout(topReplaceLayout);
+    topLayout->addWidget(m_topReplaceWidget);
+
     m_messageWidget = new QFrame;
     pal.setColor(QPalette::WindowText, creatorTheme()->color(Theme::CanceledSearchTextColor));
     m_messageWidget->setPalette(pal);
@@ -126,7 +136,7 @@ SearchResultWidget::SearchResultWidget(QWidget *parent) :
     layout->addWidget(m_messageWidget);
     m_messageWidget->setVisible(false);
 
-    m_searchResultTreeView = new Internal::SearchResultTreeView(this);
+    m_searchResultTreeView = new SearchResultTreeView(this);
     m_searchResultTreeView->setFrameStyle(QFrame::NoFrame);
     m_searchResultTreeView->setAttribute(Qt::WA_MacShowFocusRect, false);
     Aggregation::Aggregate * agg = new Aggregation::Aggregate;
@@ -138,7 +148,7 @@ SearchResultWidget::SearchResultWidget(QWidget *parent) :
     m_infoBarDisplay.setTarget(layout, 2);
     m_infoBarDisplay.setInfoBar(&m_infoBar);
 
-    m_descriptionContainer = new QWidget(topWidget);
+    m_descriptionContainer = new QWidget(topFindWidget);
     QHBoxLayout *descriptionLayout = new QHBoxLayout(m_descriptionContainer);
     m_descriptionContainer->setLayout(descriptionLayout);
     descriptionLayout->setMargin(0);
@@ -150,62 +160,61 @@ SearchResultWidget::SearchResultWidget(QWidget *parent) :
     m_searchTerm->setVisible(false);
     descriptionLayout->addWidget(m_label);
     descriptionLayout->addWidget(m_searchTerm);
-    m_cancelButton = new QToolButton(topWidget);
+    m_cancelButton = new QToolButton(topFindWidget);
     m_cancelButton->setText(tr("Cancel"));
     m_cancelButton->setToolButtonStyle(Qt::ToolButtonTextOnly);
-    connect(m_cancelButton, SIGNAL(clicked()), this, SLOT(cancel()));
-    m_searchAgainButton = new QToolButton(topWidget);
+    connect(m_cancelButton, &QAbstractButton::clicked, this, &SearchResultWidget::cancel);
+    m_searchAgainButton = new QToolButton(topFindWidget);
     m_searchAgainButton->setToolTip(tr("Repeat the search with same parameters."));
     m_searchAgainButton->setText(tr("Search again"));
     m_searchAgainButton->setToolButtonStyle(Qt::ToolButtonTextOnly);
     m_searchAgainButton->setVisible(false);
-    connect(m_searchAgainButton, SIGNAL(clicked()), this, SLOT(searchAgain()));
+    connect(m_searchAgainButton, &QAbstractButton::clicked, this, &SearchResultWidget::searchAgain);
 
-    m_replaceLabel = new QLabel(tr("Replace with:"), topWidget);
-    m_replaceTextEdit = new WideEnoughLineEdit(topWidget);
+    m_replaceLabel = new QLabel(tr("Replace with:"), m_topReplaceWidget);
+    m_replaceTextEdit = new WideEnoughLineEdit(m_topReplaceWidget);
     m_replaceTextEdit->setMinimumWidth(120);
     m_replaceTextEdit->setEnabled(false);
     m_replaceTextEdit->setTabOrder(m_replaceTextEdit, m_searchResultTreeView);
-    m_replaceButton = new QToolButton(topWidget);
+    m_replaceButton = new QToolButton(m_topReplaceWidget);
     m_replaceButton->setToolTip(tr("Replace all occurrences."));
     m_replaceButton->setText(tr("Replace"));
     m_replaceButton->setToolButtonStyle(Qt::ToolButtonTextOnly);
     m_replaceButton->setEnabled(false);
-    m_preserveCaseCheck = new QCheckBox(topWidget);
+    m_preserveCaseCheck = new QCheckBox(m_topReplaceWidget);
     m_preserveCaseCheck->setText(tr("Preserve case"));
     m_preserveCaseCheck->setEnabled(false);
 
-    if (FindPlugin * plugin = FindPlugin::instance()) {
-        m_preserveCaseCheck->setChecked(plugin->hasFindFlag(FindPreserveCase));
-        connect(m_preserveCaseCheck, SIGNAL(clicked(bool)), plugin, SLOT(setPreserveCase(bool)));
-    }
+    m_preserveCaseCheck->setChecked(Find::hasFindFlag(FindPreserveCase));
+    connect(m_preserveCaseCheck, &QAbstractButton::clicked, Find::instance(), &Find::setPreserveCase);
 
-    m_matchesFoundLabel = new QLabel(topWidget);
+    m_matchesFoundLabel = new QLabel(topFindWidget);
     updateMatchesFoundLabel();
 
-    topLayout->addWidget(m_descriptionContainer);
-    topLayout->addWidget(m_cancelButton);
-    topLayout->addWidget(m_searchAgainButton);
-    topLayout->addWidget(m_replaceLabel);
-    topLayout->addWidget(m_replaceTextEdit);
-    topLayout->addWidget(m_replaceButton);
-    topLayout->addWidget(m_preserveCaseCheck);
-    topLayout->addStretch(2);
-    topLayout->addWidget(m_matchesFoundLabel);
-    topWidget->setMinimumHeight(m_cancelButton->sizeHint().height()
-                                + topLayout->contentsMargins().top() + topLayout->contentsMargins().bottom()
-                                + topWidget->lineWidth());
-    setShowReplaceUI(false);
+    topFindLayout->addWidget(m_descriptionContainer);
+    topFindLayout->addWidget(m_cancelButton);
+    topFindLayout->addWidget(m_searchAgainButton);
+    topFindLayout->addStretch(2);
+    topFindLayout->addWidget(m_matchesFoundLabel);
+    topReplaceLayout->addWidget(m_replaceLabel);
+    topReplaceLayout->addWidget(m_replaceTextEdit);
+    topReplaceLayout->addWidget(m_replaceButton);
+    topReplaceLayout->addWidget(m_preserveCaseCheck);
+    topReplaceLayout->addStretch(2);
+    setShowReplaceUI(m_replaceSupported);
+    setSupportPreserveCase(true);
 
-    connect(m_searchResultTreeView, SIGNAL(jumpToSearchResult(SearchResultItem)),
-            this, SLOT(handleJumpToSearchResult(SearchResultItem)));
-    connect(m_replaceTextEdit, SIGNAL(returnPressed()), this, SLOT(handleReplaceButton()));
-    connect(m_replaceButton, SIGNAL(clicked()), this, SLOT(handleReplaceButton()));
+    connect(m_searchResultTreeView, &SearchResultTreeView::jumpToSearchResult,
+            this, &SearchResultWidget::handleJumpToSearchResult);
+    connect(m_replaceTextEdit, &QLineEdit::returnPressed,
+            this, &SearchResultWidget::handleReplaceButton);
+    connect(m_replaceButton, &QAbstractButton::clicked,
+            this, &SearchResultWidget::handleReplaceButton);
 }
 
 SearchResultWidget::~SearchResultWidget()
 {
-    if (m_infoBar.containsInfo(Core::Id(SIZE_WARNING_ID)))
+    if (m_infoBar.containsInfo(Id(SIZE_WARNING_ID)))
         cancelAfterSizeWarning();
 }
 
@@ -240,17 +249,18 @@ void SearchResultWidget::addResults(const QList<SearchResultItem> &items, Search
     updateMatchesFoundLabel();
     if (firstItems) {
         if (!m_dontAskAgainGroup.isEmpty()) {
-            Core::Id undoWarningId = Core::Id("warninglabel/").withSuffix(m_dontAskAgainGroup);
+            Id undoWarningId = Id("warninglabel/").withSuffix(m_dontAskAgainGroup);
             if (m_infoBar.canInfoBeAdded(undoWarningId)) {
-                Core::InfoBarEntry info(undoWarningId, tr("This change cannot be undone."),
-                                        Core::InfoBarEntry::GlobalSuppressionEnabled);
+                InfoBarEntry info(undoWarningId, tr("This change cannot be undone."),
+                                  InfoBarEntry::GlobalSuppressionEnabled);
                 m_infoBar.addInfo(info);
             }
         }
 
         m_replaceTextEdit->setEnabled(true);
         // We didn't have an item before, set the focus to the search widget or replace text edit
-        if (m_isShowingReplaceUI) {
+        setShowReplaceUI(m_replaceSupported);
+        if (m_replaceSupported) {
             m_replaceTextEdit->setFocus();
             m_replaceTextEdit->selectAll();
         } else {
@@ -261,13 +271,13 @@ void SearchResultWidget::addResults(const QList<SearchResultItem> &items, Search
     } else if (m_count <= SEARCHRESULT_WARNING_LIMIT) {
         return;
     } else {
-        Core::Id sizeWarningId(SIZE_WARNING_ID);
+        Id sizeWarningId(SIZE_WARNING_ID);
         if (!m_infoBar.canInfoBeAdded(sizeWarningId))
             return;
         emit paused(true);
-        Core::InfoBarEntry info(sizeWarningId,
-                                tr("The search resulted in more than %n items, do you still want to continue?",
-                                0, SEARCHRESULT_WARNING_LIMIT));
+        InfoBarEntry info(sizeWarningId,
+                          tr("The search resulted in more than %n items, do you still want to continue?",
+                             0, SEARCHRESULT_WARNING_LIMIT));
         info.setCancelButtonInfo(tr("Cancel"), [this]() { cancelAfterSizeWarning(); });
         info.setCustomButtonInfo(tr("Continue"), [this]() { continueAfterSizeWarning(); });
         m_infoBar.addInfo(info);
@@ -282,13 +292,10 @@ int SearchResultWidget::count() const
     return m_count;
 }
 
-QString SearchResultWidget::dontAskAgainGroup() const
-{
-    return m_dontAskAgainGroup;
-}
-
-void SearchResultWidget::setDontAskAgainGroup(const QString &group)
+void SearchResultWidget::setSupportsReplace(bool replaceSupported, const QString &group)
 {
+    m_replaceSupported = replaceSupported;
+    setShowReplaceUI(replaceSupported);
     m_dontAskAgainGroup = group;
 }
 
@@ -306,15 +313,13 @@ QString SearchResultWidget::textToReplace() const
 void SearchResultWidget::setSupportPreserveCase(bool enabled)
 {
     m_preserveCaseSupported = enabled;
+    m_preserveCaseCheck->setVisible(m_preserveCaseSupported);
 }
 
 void SearchResultWidget::setShowReplaceUI(bool visible)
 {
     m_searchResultTreeView->model()->setShowReplaceUI(visible);
-    m_replaceLabel->setVisible(visible);
-    m_replaceTextEdit->setVisible(visible);
-    m_replaceButton->setVisible(visible);
-    m_preserveCaseCheck->setVisible(visible && m_preserveCaseSupported);
+    m_topReplaceWidget->setVisible(visible);
     m_isShowingReplaceUI = visible;
 }
 
@@ -402,7 +407,7 @@ void SearchResultWidget::restart()
     m_replaceButton->setEnabled(false);
     m_searchResultTreeView->clear();
     m_count = 0;
-    Core::Id sizeWarningId(SIZE_WARNING_ID);
+    Id sizeWarningId(SIZE_WARNING_ID);
     m_infoBar.removeInfo(sizeWarningId);
     m_infoBar.enableInfo(sizeWarningId);
     m_cancelButton->setVisible(true);
@@ -425,7 +430,7 @@ void SearchResultWidget::setSearchAgainEnabled(bool enabled)
 
 void SearchResultWidget::finishSearch(bool canceled)
 {
-    Core::Id sizeWarningId(SIZE_WARNING_ID);
+    Id sizeWarningId(SIZE_WARNING_ID);
     m_infoBar.removeInfo(sizeWarningId);
     m_infoBar.enableInfo(sizeWarningId);
     m_replaceTextEdit->setEnabled(m_count > 0);
@@ -443,13 +448,13 @@ void SearchResultWidget::sendRequestPopup()
 
 void SearchResultWidget::continueAfterSizeWarning()
 {
-    m_infoBar.suppressInfo(Core::Id(SIZE_WARNING_ID));
+    m_infoBar.suppressInfo(Id(SIZE_WARNING_ID));
     emit paused(false);
 }
 
 void SearchResultWidget::cancelAfterSizeWarning()
 {
-    m_infoBar.suppressInfo(Core::Id(SIZE_WARNING_ID));
+    m_infoBar.suppressInfo(Id(SIZE_WARNING_ID));
     emit cancelled();
     emit paused(false);
 }
@@ -465,6 +470,7 @@ void SearchResultWidget::handleReplaceButton()
     // by pressing return in replace line edit
     if (m_replaceButton->isEnabled()) {
         m_infoBar.clear();
+        setShowReplaceUI(false);
         emit replaceButtonClicked(m_replaceTextEdit->text(), checkedItems(),
                                   m_preserveCaseSupported && m_preserveCaseCheck->isChecked());
     }
@@ -473,7 +479,7 @@ void SearchResultWidget::handleReplaceButton()
 void SearchResultWidget::cancel()
 {
     m_cancelButton->setVisible(false);
-    if (m_infoBar.containsInfo(Core::Id(SIZE_WARNING_ID)))
+    if (m_infoBar.containsInfo(Id(SIZE_WARNING_ID)))
         cancelAfterSizeWarning();
     else
         emit cancelled();
@@ -487,15 +493,16 @@ void SearchResultWidget::searchAgain()
 QList<SearchResultItem> SearchResultWidget::checkedItems() const
 {
     QList<SearchResultItem> result;
-    Internal::SearchResultTreeModel *model = m_searchResultTreeView->model();
-    const int fileCount = model->rowCount(QModelIndex());
+    SearchResultTreeModel *model = m_searchResultTreeView->model();
+    const int fileCount = model->rowCount();
     for (int i = 0; i < fileCount; ++i) {
-        QModelIndex fileIndex = model->index(i, 0, QModelIndex());
-        Internal::SearchResultTreeItem *fileItem = static_cast<Internal::SearchResultTreeItem *>(fileIndex.internalPointer());
-        Q_ASSERT(fileItem != 0);
+        QModelIndex fileIndex = model->index(i, 0);
+        SearchResultTreeItem *fileItem = static_cast<SearchResultTreeItem *>(fileIndex.internalPointer());
+        QTC_ASSERT(fileItem != 0, continue);
         for (int rowIndex = 0; rowIndex < fileItem->childrenCount(); ++rowIndex) {
             QModelIndex textIndex = model->index(rowIndex, 0, fileIndex);
-            Internal::SearchResultTreeItem *rowItem = static_cast<Internal::SearchResultTreeItem *>(textIndex.internalPointer());
+            SearchResultTreeItem *rowItem = static_cast<SearchResultTreeItem *>(textIndex.internalPointer());
+            QTC_ASSERT(rowItem != 0, continue);
             if (rowItem->checkState())
                 result << rowItem->item;
         }
@@ -511,4 +518,7 @@ void SearchResultWidget::updateMatchesFoundLabel()
         m_matchesFoundLabel->setText(tr("%n matches found.", 0, m_count));
 }
 
+} // namespace Internal
+} // namespace Core
+
 #include "searchresultwidget.moc"
