@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,27 +9,21 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
-#ifndef QMLJSEDITOR_H
-#define QMLJSEDITOR_H
+#pragma once
 
 #include "qmljseditor_global.h"
 
@@ -67,15 +61,17 @@ class QmlJSEditorWidget : public TextEditor::TextEditorWidget
 public:
     QmlJSEditorWidget();
 
-    void finalizeInitialization();
+    void finalizeInitialization() override;
 
     QmlJSEditorDocument *qmlJsEditorDocument() const;
 
     QModelIndex outlineModelIndex();
 
     TextEditor::AssistInterface *createAssistInterface(TextEditor::AssistKind assistKind,
-                                                        TextEditor::AssistReason reason) const;
-public slots:
+                           TextEditor::AssistReason reason) const override;
+
+    void inspectElementUnderCursor() const;
+
     void findUsages();
     void renameUsages();
     void showContextPane();
@@ -84,7 +80,7 @@ signals:
     void outlineModelIndexChanged(const QModelIndex &index);
     void selectedElementsChanged(QList<QmlJS::AST::UiObjectMember*> offsets,
                                  const QString &wordAtCursor);
-private slots:
+private:
     void modificationChanged(bool);
 
     void jumpToOutlineElement(int index);
@@ -96,22 +92,21 @@ private slots:
 
     void semanticInfoUpdated(const QmlJSTools::SemanticInfo &semanticInfo);
 
-    void performQuickFix(int index);
     void updateCodeWarnings(QmlJS::Document::Ptr doc);
 
 protected:
-    void contextMenuEvent(QContextMenuEvent *e);
-    bool event(QEvent *e);
-    void wheelEvent(QWheelEvent *event);
-    void resizeEvent(QResizeEvent *event);
-    void scrollContentsBy(int dx, int dy);
-    void applyFontSettings();
+    void contextMenuEvent(QContextMenuEvent *e) override;
+    bool event(QEvent *e) override;
+    void wheelEvent(QWheelEvent *event) override;
+    void resizeEvent(QResizeEvent *event) override;
+    void scrollContentsBy(int dx, int dy) override;
+    void applyFontSettings() override;
     void createToolBar();
     TextEditor::TextEditorWidget::Link findLinkAt(const QTextCursor &cursor,
                                                       bool resolveTarget = true,
-                                                      bool inNextSplit = false);
-    QString foldReplacementText(const QTextBlock &block) const;
-    void onRefactorMarkerClicked(const TextEditor::RefactorMarker &marker);
+                                                      bool inNextSplit = false) override;
+    QString foldReplacementText(const QTextBlock &block) const override;
+    void onRefactorMarkerClicked(const TextEditor::RefactorMarker &marker) override;
 
 private:
     bool isClosingBrace(const QList<QmlJS::Token> &tokens) const;
@@ -130,14 +125,10 @@ private:
     QModelIndex m_outlineModelIndex;
     QmlJS::ModelManagerInterface *m_modelManager;
 
-    TextEditor::QuickFixOperations m_quickFixes;
-
     QmlJS::IContextPane *m_contextPane;
     int m_oldCursorPosition;
 
     FindReferences *m_findReferences;
-
-    bool m_firstSementicInfo;
 };
 
 
@@ -148,7 +139,7 @@ class QmlJSEditor : public TextEditor::BaseTextEditor
 public:
     QmlJSEditor();
 
-    bool isDesignModePreferred() const;
+    bool isDesignModePreferred() const override;
 };
 
 class QmlJSEditorFactory : public TextEditor::TextEditorFactory
@@ -161,5 +152,3 @@ public:
 
 } // namespace Internal
 } // namespace QmlJSEditor
-
-#endif // QMLJSEDITOR_H
