@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -118,6 +113,8 @@ private Q_SLOTS:
     void unicodeIdentifier();
 
     void inAlignas();
+
+    void memberAccessAsTemplate();
 };
 
 void tst_FindUsages::dump(const QList<Usage> &usages) const
@@ -1018,5 +1015,66 @@ void tst_FindUsages::inAlignas()
     QCOMPARE(find.usages()[1].col, 15);
 }
 
+void tst_FindUsages::memberAccessAsTemplate()
+{
+    const QByteArray src = "\n"
+            "struct Foo {};\n"
+            "struct Bar {\n"
+            "    template <typename T>\n"
+            "    T *templateFunc() { return 0; }\n"
+            "};\n"
+            "struct Test {\n"
+            "    Bar member;\n"
+            "    void testFunc();\n"
+            "};\n"
+            "void Test::testFunc() {\n"
+            "    member.templateFunc<Foo>();\n"
+            "}\n";
+
+    Document::Ptr doc = Document::create("memberAccessAsTemplate");
+    doc->setUtf8Source(src);
+    doc->parse();
+    doc->check();
+
+    QVERIFY(doc->diagnosticMessages().isEmpty());
+    QCOMPARE(doc->globalSymbolCount(), 4U);
+
+    Snapshot snapshot;
+    snapshot.insert(doc);
+
+    {   // Test "Foo"
+        Class *c = doc->globalSymbolAt(0)->asClass();
+        QVERIFY(c);
+        QCOMPARE(c->name()->identifier()->chars(), "Foo");
+
+        FindUsages find(src, doc, snapshot);
+        find(c);
+        QCOMPARE(find.usages().size(), 2);
+        QCOMPARE(find.usages()[0].line, 1);
+        QCOMPARE(find.usages()[0].col, 7);
+        QCOMPARE(find.usages()[1].line, 11);
+        QCOMPARE(find.usages()[1].col, 24);
+    }
+
+    {   // Test "templateFunc"
+        Class *c = doc->globalSymbolAt(1)->asClass();
+        QVERIFY(c);
+        QCOMPARE(c->name()->identifier()->chars(), "Bar");
+        QCOMPARE(c->memberCount(), 1U);
+
+        Template *f = c->memberAt(0)->asTemplate();
+        QVERIFY(f);
+        QCOMPARE(f->name()->identifier()->chars(), "templateFunc");
+
+        FindUsages find(src, doc, snapshot);
+        find(f);
+        QCOMPARE(find.usages().size(), 2);
+        QCOMPARE(find.usages()[0].line, 4);
+        QCOMPARE(find.usages()[0].col, 7);
+        QCOMPARE(find.usages()[1].line, 11);
+        QCOMPARE(find.usages()[1].col, 11);
+    }
+}
+
 QTEST_APPLESS_MAIN(tst_FindUsages)
 #include "tst_findusages.moc"
